@@ -7,10 +7,9 @@ from typing import Tuple
 from ..utils import almost_equal
 from .Complex import Complex
 from .Vector import Vector
-from .Field import Field, DefaultRealField
+from .Field import Field, RealField
 from ..utils import areinstances, check_foreach, isoneof
 from .Span import Span
-# TODO remove hardcoded solution vec and add it as parameter to solve with
 
 
 class MatrixOperationType(enum.Enum):
@@ -45,7 +44,7 @@ class Matrix:
         return Matrix([[v] for v in vec], vec.field)
 
     @staticmethod
-    def fromVectors(vecs: list[Vector]) -> Matrix:
+    def from_vectors(vecs: list[Vector]) -> Matrix:
         """Create a Matrix from a list of Vectors, the matrix will have the same number of columns as the number of vectors and the same number of rows as the number of elements in a vector
 
         Args:
@@ -63,7 +62,7 @@ class Matrix:
         if not check_foreach(vecs, lambda v: v.field == vecs[0].field):
             raise ValueError("vectors are not over the same field")
         mat: list[list[Any]] = []
-        for i in range(vecs[0].length):
+        for i in range(len(vecs[0])):
             mat.append([])
             for j in range(len(vecs)):
                 mat[i].append(vecs[j][i])
@@ -83,13 +82,13 @@ class Matrix:
     #                    for row in matrix_string.split("\n")])
 
     @staticmethod
-    def random(rows: int, cols: int, f: Field = DefaultRealField, min: float = -10, max: float = 10, ) -> Matrix:
+    def random(rows: int, cols: int, f: Field = RealField(), min: float = -10, max: float = 10, ) -> Matrix:
         """Create a random Matrux acording to params
 
         Args:
             rows (int): the amount of rows in matrix
             cols (int): the amount of columns in matrix
-            f (Field, optional): the field to create the matrix over. Defaults to DefaultRealField.
+            f (Field, optional): the field to create the matrix over. Defaults to RealField().
             min (float, optional): the minimum value from the field. Defaults to -10.
             max (float, optional): the maximum value from the field. Defaults to 10.
 
@@ -122,12 +121,12 @@ class Matrix:
             arr[i][i] = 1
         return Matrix(arr)
 
-    def __init__(self, mat: list[list[Any]], field: Field = DefaultRealField) -> None:
+    def __init__(self, mat: list[list[Any]], field: Field = RealField()) -> None:
         """Create a Matrix
 
         Args:
             mat (list[list[Any]]): a list of lists of elements (2D array)
-            field (Field, optional): the field that the elements of the matrix is over. Defaults to DefaultRealField.
+            field (Field, optional): the field that the elements of the matrix is over. Defaults to RealField().
 
         Raises:
             TypeError: will rise if mat is not a list of lists
@@ -141,15 +140,15 @@ class Matrix:
 
     @property
     def kernel(self) -> Union[Vector, Span]:
-        solution = self.solve(Vector.fromSize(len(self), self.field))
+        solution = self.solve(Vector.from_size(len(self), self.field))
         if solution is None:
             return Vector([0 for _ in range(len(self))])
         return solution
 
     @property
     def image(self) -> list[Vector]:
-        # TODO implement image calculation
-        pass
+        from .VectorSpace import VectorSpace
+        return VectorSpace(self.field).standard_basis() - self.kernel
 
     @property
     def rank(self) -> int:
@@ -158,7 +157,7 @@ class Matrix:
         rank = 0
         for i in range(min(len(tmp), len(tmp[0]))):
             for v in tmp[i]:
-                if v != tmp.field._zero:
+                if v != tmp.field.zero:
                     rank += 1
                     break
 
@@ -228,10 +227,12 @@ class Matrix:
         # TODO implement chain basis calculation
         pass
 
-    # @property
-    # def characteristic_polynomial(self) -> SimplePolynomial.SimplePolynomial:
-    #     # TODO implement characteristic polynomial calculation
-    #     pass
+    @property
+    def characteristic_polynomial(self):
+        from ..la2 import PolynomialSimple
+        if not self.is_square:
+            raise ValueError("Matrix must be square")
+        return (Matrix.id_matrix(len(self))*PolynomialSimple.fromString("x")-self).determinant
 
     # @property
     # def minimal_polynomial(self) -> SimplePolynomial.SimplePolynomial:
@@ -264,7 +265,7 @@ class Matrix:
             raise ValueError("Index out of range")
         if not isinstance(value, list):
             raise TypeError("Value must be a list")
-        # TODO validate all items in list are of same type of this matrix
+        # FIXME validate all items in list are of same type of this matrix
         self.__matrix[index] = value
 
     def __str__(self, turnc: int = 2) -> str:
@@ -550,10 +551,14 @@ class Matrix:
         def comparer(a: list[float], b: list[float]) -> bool:
             def first_not_zero_index(row: list[float]) -> int:
                 for i in range(len(row)):
-                    if row[i] != self.field._zero:
+                    if row[i] != self.field.zero:
                         break
                 return i
-            return -1 if first_not_zero_index(a) > first_not_zero_index(b) else 1
+            a_index = first_not_zero_index(a)
+            b_index = first_not_zero_index(b)
+            if a_index == b_index:
+                return 0
+            return -1 if a_index > b_index else 1
 
         self.__matrix = sorted(
             self.__matrix, key=functools.cmp_to_key(comparer), reverse=True)
@@ -626,6 +631,14 @@ class Matrix:
             return self, None
         return Matrix([[self[i][j] for j in range(index)] for i in range(len(self))], self.field), Matrix([[self[i][j] for j in range(index, len(self[0]))] for i in range(len(self))], self.field)
 
+    def duplicate(self) -> Matrix:
+        mat: list[list[Any]] = []
+        for i, row in enumerate(self):
+            mat.append([])
+            for v in row:
+                mat[i].append(v)
+        return Matrix(mat, self.field)
+
     def gaussian_elimination_with(self, solve_with: Union[Vector, Matrix] = None) -> Tuple[Matrix, Union[None, Matrix]]:
         """will perform gaussian elimination on the matrix and return the result in a new matrix and the solution if solve_with is given
 
@@ -639,7 +652,7 @@ class Matrix:
             Tuple[Matrix, Union[None, Matrix]]: the result of the gaussian elimination and the solution if solve_with is given
         """
         SPLIT_INDEX = len(self[0])
-        res = self
+        res = self.duplicate()
         if solve_with is not None:
             if not (isoneof(solve_with, [Vector, Matrix]) or solve_with is None):
                 raise TypeError("can only solve with a Vector or Matrix")
@@ -649,7 +662,7 @@ class Matrix:
         # performe gaussian elimination
         for curr_row_index in range(len(res)):
             # find lead value for current row
-            for lead_index in range(curr_row_index, SPLIT_INDEX):
+            for lead_index in range(SPLIT_INDEX):
                 if res[curr_row_index][lead_index] != 0:
                     break
             else:
@@ -677,6 +690,7 @@ class Matrix:
                         MOT.ROW_ADDITION, next_row_index, curr_row_index)
                     res.apply_operation(
                         MOT.ROW_MULTIPLICATION, next_row_index, -row_multiplyer)
+        res.reorgenize_rows()
         return res.split(SPLIT_INDEX)
 
     def gaussian_elimination(self) -> Matrix:
@@ -699,6 +713,20 @@ class Matrix:
         return Vector(arr)
 
     def solve(self, vec: Vector) -> Union[None, Vector, Span]:
+        """will solve the matrix with the given vector and return the solution
+
+        Args:
+            vec (Vector): the vector to solve with
+
+        Raises:
+            TypeError: will rise if vec is not a Vector
+            NotImplementedError: will rise if there is a span of solutions and vector!=V0 ,WILL BE IMPLEMENTED LATER
+
+        Returns:
+            None: if there is no solution
+            Vector: if there is a single solution
+            Span: if there are multiple solutions
+        """
         if not isinstance(vec, Vector):
             raise TypeError("Matrix must be solved for a vector")
         result_matrix, sol = self.gaussian_elimination_with(vec)
@@ -710,25 +738,27 @@ class Matrix:
 
         # if there is a row in result matrix which has all zeros and solution vector doesnt has a zero at that row there is no solution
         for i, row in enumerate(result_matrix):
-            if (not Vector(row).has_no_zero) and sol[i] != 0:
+            # if (not Vector(row).has_no_zero) and sol[i] != 0:
+            #     return None
+            has_no_zero = check_foreach(row, lambda x: x != 0)
+            if has_no_zero and sol[i] != 0:
                 return None
-
         # otherwise there is a span of solutions
 
         # solve for V0
-        if sol == Vector.fromSize(len(result_matrix), result_matrix.field):
+        if sol == Vector.from_size(len(result_matrix), result_matrix.field):
             def get_solutions_from_columns(m: Matrix) -> list[Vector]:
                 def sumrows(m: Matrix) -> list[int]:
                     sumrow = [0 for _ in range(len(m[0]))]
                     for i in range(m.rank):
                         row = m[i]
                         for j, v in enumerate(row):
-                            if v != m.field._zero:
+                            if v != m.field.zero:
                                 sumrow[j] += 1
                     return sumrow
                 res = []
                 for i2, v in enumerate(sumrows(m)):
-                    if v == m.field._zero:
+                    if v == m.field.zero:
                         res.append(Vector.e(i2, len(m), m.field))
                 return res
 
@@ -738,9 +768,9 @@ class Matrix:
                 for row_index in range(m.rank):
                     row = m[row_index]
                     for candidate_index, candidate in enumerate(row):
-                        if candidate != m.field._zero:
+                        if candidate != m.field.zero:
                             for validator_index in range(candidate_index+1, len(row)):
-                                if row[validator_index] != m.field._zero:
+                                if row[validator_index] != m.field.zero:
                                     if validator_index not in value_depends_on_key:
                                         value_depends_on_key[validator_index] = set(
                                         )
@@ -749,12 +779,12 @@ class Matrix:
                             break
                 for key in value_depends_on_key:
                     tmp = [0 for _ in range(len(m[0]))]
-                    tmp[key] = m.field._one
+                    tmp[key] = m.field.one
                     for index in value_depends_on_key[key]:
-                        tmp[index] = -m.field._one
+                        tmp[index] = -m.field.one
                     res.append(Vector(tmp))
                 return res
-
+            # FIXME fix matrix.solve to support vectors who are not V0
             return Span(get_solutions_from_columns(result_matrix)+get_solutions_from_rows(result_matrix))
 
         raise NotImplementedError(
